/*
 * Copyright (c) 2001, Swedish Institute of Computer Science.
 * All rights reserved. 
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions 
 * are met: 
 *
 * 1. Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer. 
 *
 * 2. Redistributions in binary form must reproduce the above copyright 
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the distribution. 
 *
 * 3. Neither the name of the Institute nor the names of its contributors 
 *    may be used to endorse or promote products derived from this software 
 *    without specific prior written permission. 
 *
 * THIS SOFTWARE IS PROVIDED BY THE INSTITUTE AND CONTRIBUTORS ``AS IS'' AND 
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE INSTITUTE OR CONTRIBUTORS BE LIABLE 
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS 
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) 
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT 
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY 
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF 
 * SUCH DAMAGE. 
 *
 * This file is part of the lwIP TCP/IP stack.
 * 
 * Author: Adam Dunkels <adam@sics.se>
 *
 * $Id: sys.h,v 1.3 2001/02/21 19:52:27 adam Exp $
 */
#ifndef __LWIP_SYS_H__
#define __LWIP_SYS_H__

#include "arch/sys_arch.h"

/* must be called before anthing else */
void sys_init(void);

/* IRQ functions. */
typedef void (* sys_irq_handler)(int fd, void *data);
void sys_irq(int irqnum, sys_irq_handler h, void *data);


/* Timeout functions. */
typedef void (* sys_timeout_handler)(void *data);
void sys_timeout(unsigned int msecs, sys_timeout_handler h, void *data);

/* Semaphore functions. */
sys_sem_t sys_sem_new(int count);
void sys_sem_signal(sys_sem_t sem);
void sys_sem_wait(sys_sem_t sem);
void sys_sem_free(sys_sem_t sem);

/* Mailbox functions. */
sys_mbox_t sys_mbox_new(void);
void sys_mbox_post(sys_mbox_t mbox, void *msg);
void sys_mbox_fetch(sys_mbox_t mbox, void **msg);
void sys_mbox_free(sys_mbox_t mbox);


/* Thread functions. */
sys_thread_t sys_thread_new(void *(* function)(void *arg), void *arg);

/* A null loop, timers and irqs can be fired. */
void sys_main(void);

/* The following functions are used only in Unix code, and
   can be omitted when porting the stack. */

/* Returns the current time in microseconds. */
unsigned long sys_now(void);

#endif /* __LWIP_SYS_H__ */



