/*
 * Copyright (c) 2001, Swedish Institute of Computer Science.
 * All rights reserved. 
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions 
 * are met: 
 *
 * 1. Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer. 
 *
 * 2. Redistributions in binary form must reproduce the above copyright 
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the distribution. 
 *
 * 3. Neither the name of the Institute nor the names of its contributors 
 *    may be used to endorse or promote products derived from this software 
 *    without specific prior written permission. 
 *
 * THIS SOFTWARE IS PROVIDED BY THE INSTITUTE AND CONTRIBUTORS ``AS IS'' AND 
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE INSTITUTE OR CONTRIBUTORS BE LIABLE 
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS 
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) 
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT 
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY 
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF 
 * SUCH DAMAGE. 
 *
 * This file is part of the lwIP TCP/IP stack.
 * 
 * Author: Adam Dunkels <adam@sics.se>
 *
 * $Id: test_list.c,v 1.4 2001/02/15 09:42:30 adam Exp $
 */

#include <stdio.h>
#include <stdlib.h>

#include "lwip/debug.h"
#include "lwip/list.h"

#define SIZE 20

/*-----------------------------------------------------------------------------------*/
void
print(void *str)
{
  printf("%s\n", (char *)str);
}
/*-----------------------------------------------------------------------------------*/
int
main(int argc, char **argv)
{
  struct list *l;
  int i;
  char *buf;
  char *hej;
  
  l = list_new(SIZE + 1);
  list_map(l, print);
  
  hej = malloc(200);
  sprintf(hej, "Hej!");
  if(list_push(l, hej) == 0) {
    printf("Could not push %s\n", hej);
  }

  for(i = 0; i < SIZE; i++) {
    buf = malloc(200);
    sprintf(buf, "%d", i);
    if(list_push(l, buf) == 0) {
      printf("Could not push %s\n", buf);
      free(buf);
    }                                 
  }

  list_map(l, print);
  printf("\n");
  list_remove(l, hej);

  
  if(list_push(l, hej) == 0) {
    printf("Could not push %s\n", hej);
  }
  list_remove(l, hej);
  
  for(i = 0; i < SIZE + 2; i++) {
    buf = list_pop(l);
    free(buf);
  }


  if(list_push(l, hej) == 0) {
    printf("Could not push %s\n", hej);
  }

  list_map(l, print);
  printf("\n");
  list_remove(l, hej);
  
  for(i = 0; i < SIZE; i++) {
    buf = malloc(200);
    sprintf(buf, "%d", i);
    if(list_push(l, buf) == 0) {
      printf("Could not push %s\n", buf);
      free(buf);
    }                                 
  }

  list_map(l, print);
  
  for(i = 0; i < SIZE + 2; i++) {
    buf = list_pop(l);
    free(buf);
  }

  list_map(l, print);

  printf("%d\n", (unsigned int)-1);
  
  exit(0);
}
/*-----------------------------------------------------------------------------------*/
